@@ -3,9 +3,10 @@ from copy import deepcopy
 
 from .algorithm import Algorithm
 from .operators import RandomGenerator, SimulatedBinaryCrossover, \
-    PmMutator, TournamentSelector, EpsilonDominance, nondominated_truncate
+    PmMutator, TournamentSelector, EpsilonDominance, nondominated_truncate, crowding_distance
 from .population import Population
 from .problem import Problem
+from .archive import Archive
 
 
 class GeneralEvolutionaryAlgorithm(Algorithm):
@@ -13,6 +14,17 @@ class GeneralEvolutionaryAlgorithm(Algorithm):
 
     def __init__(self, problem: Problem, name="General Evolutionary Algorithm"):
         super().__init__(problem, name)
+
+        self.parameters_length = len(self.problem.parameters)
+
+        self.options.declare(name='n_iterations', default=50, lower=1,
+                             desc='Maximum evaluations')
+        self.options.declare(name='max_population_number', default=100, lower=1,
+                             desc='max_population_number')
+        self.options.declare(name='max_population_size', default=100, lower=1,
+                             desc='Maximal number of individuals in population')
+        self.features = dict()
+
         self.problem = problem
 
         self.generator = None
@@ -25,16 +37,14 @@ class GeneralEvolutionaryAlgorithm(Algorithm):
 
         # set random generator
         self.generator = RandomGenerator(self.problem.parameters)
-        # self.generator.init(10)
 
-    def gen_initial_population(self, is_archive=False):
+    def add_features(self, individuals):
+        for individual in individuals:
+            individual.features = self.features.copy()
+
+    def gen_initial_population(self):
         individuals = self.generator.generate()
-
-        # create population
-        if is_archive:
-            population = Population(individuals, individuals)
-        else:
-            population = Population(individuals)
+        population = Population(individuals)
 
         # append to population
         self.problem.populations.append(population)
@@ -49,31 +59,22 @@ class GeneralEvolutionaryAlgorithm(Algorithm):
 
 class GeneticAlgorithm(GeneralEvolutionaryAlgorithm):
 
-    def __init__(self, problem: Problem, name="General Evolutionary Algorithm"):
+    def __init__(self, problem: Problem, name="General Genetic-based Algorithm"):
         super().__init__(problem, name)
-        self.parameters_length = len(self.problem.parameters)
-
-        self.options.declare(name='n_iterations', default=50, lower=1,
-                             desc='Maximum evaluations')
-        self.options.declare(name='max_population_number', default=100, lower=1,
-                             desc='max_population_number')
-        self.options.declare(name='max_population_size', default=100, lower=1,
-                             desc='Maximal number of individuals in population')
-        self.features = dict()
-
-    def add_features(self, individuals):
-        for individual in individuals:
-            individual.features = self.features.copy()
 
     def generate(self, parents, archive=None):
         offsprings = []
-        while len(offsprings) < self.population_size:
+        offsprings.extend(parents)
+        while len(offsprings) < 2 * self.population_size:
             parent1 = self.selector.select(parents)
 
             repeat = True
             while repeat:
                 if archive:
-                    parent2 = self.selector.select(archive)
+                    if len(archive) <= 1:
+                        parent2 = self.selector.select(parents)
+                    else:
+                        parent2 = archive.rand_choice()
                 else:
                     parent2 = self.selector.select(parents)
 
@@ -136,7 +137,6 @@ class NSGAII(GeneticAlgorithm):
 
         # non-dominated sort of individuals
         self.selector.fast_nondominated_sorting(population.individuals)
-        # self.selector.crowding_distance(population.individuals)
 
         t_s = time.time()
         self.problem.logger.info(
@@ -153,12 +153,8 @@ class NSGAII(GeneticAlgorithm):
             self.evaluate(offsprings)
             self.add_features(offsprings)
 
-            # add the parents to the offsprings
-            offsprings.extend(deepcopy(population.individuals))
-
             # make the pareto dominance calculation and calculating the crowding distance
             self.selector.fast_nondominated_sorting(offsprings)
-            # self.selector.crowding_distance(offsprings)
             population.individuals = nondominated_truncate(offsprings, self.population_size)
 
         t = time.time() - t_s
@@ -166,6 +162,19 @@ class NSGAII(GeneticAlgorithm):
 
 
 class EpsMOEA(GeneticAlgorithm):
+    """
+    EpsMOEA is proposed by Deb et al[1] which is a efficient algorithm with less computer time because it does not
+    contain a sort process for non-dominate set. The searching space are divided into several hyper-boxes under
+    box dominance concept, when two solutions in same box, only one solution can survive in the hyperbox, EpsMOEA choose
+    the solution nearer to the hyperbox corner.
+
+    .. Ref::
+
+    [1] K. Deb, M. Mohan, S. Mishra, Evaluating the epsilon-domination based multiobjective evolutionary algorithm for
+        a quick computation of Pareto-optimal solutions, Evolutionary Computation 13(4) (2005) 501 - 525.
+    [2] Siwei Jiang, Zhihua Cai: Enhance the Convergence and Diversity for eps-MOPSO by Uniform Design and
+                                 Minimum Reduce Hypervolume
+    """
 
     def __init__(self, problem: Problem, name="EpsMOEA Algorithm"):
         super().__init__(problem, name)
@@ -184,69 +193,47 @@ class EpsMOEA(GeneticAlgorithm):
 
     def run(self):
         # set random generator
-        self.generator = RandomGenerator(self.problem.parameters)  # the same as in the case of NSGA-II
+        self.generator = RandomGenerator(self.problem.parameters)
         self.generator.init(self.options['max_population_size'])
 
         # set crossover
         self.crossover = SimulatedBinaryCrossover(self.problem.parameters, self.options['prob_cross'])
         self.mutator = PmMutator(self.problem.parameters, self.options['prob_mutation'])
 
-        # Part A: non-dominated sort of individuals
-        # -----
-        selector_pareto = TournamentSelector(self.problem.parameters)
+        # one individual is selected from the archive and one from the population to create a new individual
         self.selector = TournamentSelector(self.problem.parameters)
-        # the same as in the case of NSGA - ii
-        # this operator is used to generate the new individuals
 
         # create initial population and evaluate individuals
-        population = self.gen_initial_population(True)  # archiving True
+        population = self.gen_initial_population()
+
+        # an archive to collect the eps-dominated solutions
+        self.problem.archive = Archive(dominance=EpsilonDominance(epsilons=self.options['epsilons']))
+
         self.evaluate(population.individuals)
         self.add_features(population.individuals)
-        selector_pareto.fast_nondominated_sorting(population.individuals)
-        selector_pareto.crowding_distance(population.individuals)
 
-        # Part B: eps-dominated sort of the individuals with archiving
-        # -----
-        selector_epsdom = TournamentSelector(self.problem.parameters,
-                                             dominance=EpsilonDominance, epsilons=self.options['epsilons'])
-        selector_epsdom.fast_nondominated_sorting(population.archives)
-        selector_epsdom.crowding_distance(population.archives)
+        # archiving the eps-dominating solutions
+        for individual in population.individuals:
+            self.problem.archive.add(individual)
 
         t_s = time.time()
         self.problem.logger.info(
             "Eps-MOEA: {}/{}".format(self.options['max_population_number'], self.population_size))
 
-        # optimization
         for it in range(self.options['max_population_number']):
             # generate and evaluate the next generation
-            children = self.generate(population.individuals, population.archives)
+            offspring = self.generate(population.individuals, archive=self.problem.archive)
 
-            population = Population(children)
+            self.evaluate(offspring)
+            self.add_features(offspring)
+
+            # pop-acceptance procedure, the dominating offsprings will be preserved in the population and  in the
+            # archive
+            population = Population(population.individuals)  # make a new population from the previous population
             self.problem.populations.append(population)
-            self.evaluate(children)
-            self.add_features(children)
-
-            arch_child = deepcopy(children)
-            # PART A
-            # non-dominated sorting of the newly generated and the older guys like in NSGA-ii
-            # add the parents to the offsprings
-            children.extend(deepcopy(population.individuals))
-
-            # non-dominated truncate on the guys
-            self.selector.fast_nondominated_sorting(children)
-            selector_pareto.crowding_distance(children)
-
-            parents = sorted(set(children),
-                             key=lambda x: (x.features['front_number'], -x.features['crowding_distance']))
-            child = parents[:self.population_size]  # truncate
-
-            # eps dominated truncate on the guys
-            selector_epsdom.fast_nondominated_sorting(arch_child)
-            selector_epsdom.crowding_distance(arch_child)
-
-            arch_parents = sorted(set(arch_child),
-                                  key=lambda x: (x.features['front_number'], -x.features['crowding_distance']))
-            population.archives = arch_parents[:self.population_size]  # truncate
+            for individual in offspring:
+                self.selector.pop_acceptance(population.individuals, individual)    # pareto dominated solutions
+                self.problem.archive.add(individual)                                # archived solutions
 
         t = time.time() - t_s
         self.problem.logger.info("Eps-MOEA: {} s".format(t))
