@@ -3,6 +3,7 @@ from abc import ABCMeta
 from .individual import Individual
 from math import inf
 
+
 class Job(metaclass=ABCMeta):
     def __init__(self, problem, population=None):
         self.problem = problem
@@ -25,13 +26,9 @@ class Job(metaclass=ABCMeta):
         # problem cost function evaluate only in that case when the problem fits the constraints
         try:
             costs = self.problem.surrogate.evaluate(individual)
-            if isinstance(costs, list):
-                individual.costs = costs
-                if self.problem is not None:
-                    individual.calc_signed_costs(self.problem.signs)  # the idea is to make this conversion only once
-                #     individual.signs = self.problem.signs
-            else:
-                raise AssertionError("Costs type must be list.")
+            individual.costs = costs
+            if self.problem is not None:
+                individual.calc_signed_costs(self.problem.signs)  # the idea is to make this conversion only once
 
             # set evaluated
             individual.state = individual.State.EVALUATED
@@ -48,10 +45,3 @@ class Job(metaclass=ABCMeta):
 
         # info
         individual.info["finish_time"] = time.time()
-
-    def evaluate_scalar(self, x):
-        # simple individual
-        individual = Individual(x)
-        self.evaluate(individual)
-
-        return individual.costs[0]
