@@ -15,8 +15,11 @@ from copy import deepcopy
 
 EPSILON = sys.float_info.epsilon
 
+# global constants to p and q dominates
+QDOM = 1
+PDOM = 2
+
 
-# TODO: Seems to be useless
 class Operator(ABC):
 
     def __init__(self):
@@ -34,6 +37,7 @@ class Evaluator(Operator):
         super().__init__()
         self.algorithm = algorithm
         self.individuals = []
+        self.job = Job(self.algorithm.problem)
 
     def add(self, individual):
         self.individuals.append(individual)
@@ -47,26 +51,23 @@ class Evaluator(Operator):
         else:
             self.evaluate_serial(individuals)
 
-        n_failed = 0
-        # for individual in individuals:
-        #     # if individual.state == Individual.State.FAILED:
-        #     #     n_failed += 1
-        #     #     individuals.remove(individual)   # TODO: it is not correct, nsga2 handles this case in paretodominance
-        #     individual.calc_signed_costs(self.problem.signs) # the idea is to make this conversion only once
-
     def evaluate_serial(self, individuals: list):
-        job = Job(self.algorithm.problem)
         for individual in individuals:
             if individual.state == Individual.State.EMPTY:
-                job.evaluate(individual)
+                individual.costs.append(self.job.evaluate(individual))
 
     def evaluate_parallel(self, individuals: list):
         # simple parallel loop
-        job = Job(self.algorithm.problem)
         Parallel(n_jobs=self.algorithm.options["max_processes"], verbose=1, require='sharedmem')(
-            delayed(job.evaluate)(individual)
+            delayed(self.job.evaluate)(individual)
             for individual in individuals)
 
+    def evaluate_scalar(self, vector):
+        individual = Individual(vector)
+        self.algorithm.problem.populations[-1].individuals.append(individual)
+        self.job.evaluate(individual)
+        return individual.costs[0]
+
 
 class GradientEvaluator(Evaluator):
 
@@ -83,14 +84,23 @@ class GradientEvaluator(Evaluator):
             individual.children.append(Individual(vector))
             individual.children[-1].parents.append(individual)
 
+        self.to_evaluate.append(individual)
         self.to_evaluate.extend(individual.children)
 
     def evaluate(self, individuals):
-        super().evaluate(individuals)
         for individual in individuals:
             self.add(individual)
         self.run()
 
+    def evaluate_scalar(self, x):
+        individual = Individual(x)
+
+        self.add(individual)
+        self.run()
+        self.algorithm.problem.populations[-1].individuals.append(individual)
+        self.to_evaluate = []
+        return individual.costs[0]
+
     def run(self):
         n_params = len(self.individuals[0].vector)
         super().evaluate(self.to_evaluate)
@@ -105,29 +115,42 @@ class GradientEvaluator(Evaluator):
         self.to_evaluate = []
 
 
-class RichardsonGradientEvaluator(Operator):
+class WorstCaseEvaluator(Evaluator):
 
-    def __init__(self):
-        pass
+    def __init__(self, algorithm):
+        super().__init__(algorithm)
+        self.to_evaluate = []
 
-    # def evaluate_gradient_richardson(self, population, individual):
-    #     x0 = individual.vector
-    #     gradient = [0] * len(x0)
-    #
-    #     h = 1e-6
-    #     job = Job(self.problem, population)
-    #     y = job.evaluate_scalar(x0)
-    #     for i in range(len(x0)):
-    #         x = x0.copy()
-    #         x[i] += h
-    #         y_h = job.evaluate_scalar(x)
-    #         d_0_h = gradient[i] = (y_h - y) / h
-    #         x[i] += h
-    #         y_2h = job.evaluate_scalar(x)
-    #         d_0_2h = (y_2h - y) / 2 / h
-    #         gradient[i] = (4 * d_0_h - d_0_2h) / 3
-    #
-    #     return gradient
+    def add(self, individual):
+        parameters = self.algorithm.problem.parameters
+        for i in range(len(individual.vector)):
+            parameter = parameters[i]
+            for sign in [-1, 1]:
+                vector = individual.vector.copy()
+                vector[i] += sign * parameter['tol'] * vector[i]
+                individual.children.append(Individual(vector))
+                individual.children[-1].parents.append(individual)
+
+        self.to_evaluate.extend(individual.children)
+
+    def evaluate(self, individuals):
+        super().evaluate(individuals)
+        for individual in individuals:
+            self.add(individual)
+        self.run()
+
+    def evaluate_scalar(self, x):
+        parent_individual = Individual(x)
+        parent_individual.costs.append(self.job.evaluate(parent_individual))
+        self.add(parent_individual)
+        for individual in self.to_evaluate:
+            individual.costs.append(self.job.evaluate(individual))
+        self.algorithm.problem.populations[-1].individuals.append(parent_individual)
+        self.to_evaluate = []
+        return parent_individual.costs[0]
+
+    def run(self):
+        super().evaluate(self.to_evaluate)
 
 
 class Generator(Operator):
@@ -613,7 +636,7 @@ class FireflyMutator(SwarmMutator):
         dominates = True
 
         for i in range(len(current.costs)):
-            if other.signs[i] > current.signs[i]:
+            if other.costs_signed[i] > current.costs_signed[i]:
                 dominates = False
 
         return dominates
@@ -716,11 +739,7 @@ class Dominance(ABC):
 
 class EpsilonDominance(Dominance):
     """
-    Epsilon dominance.
-
-        Similar to Pareto dominance except if the two solutions are contained
-        within the same epsilon-box, the solution closer to the optimal corner
-        or the box is preferred.
+    Epsilon dominance comparator divides the number
     """
 
     def __init__(self, epsilons):
@@ -731,50 +750,17 @@ class EpsilonDominance(Dominance):
         else:
             self.epsilons = [epsilons]
 
-    def same_box(self, p, q):
-        # first check constraint violation
-        if p.feasible != q.feasible:
-            if p.feasible == 0:
-                return False
-            elif q.feasible == 0:
-                return False
-            elif p.feasible < q.feasible:
-                return False
-            elif q.feasible < p.feasible:
-                return False
-
-        # then use epsilon dominance on the objectives
-        dominate1 = False
-        dominate2 = False
-
-        for i in range(len(p.costs)):
-            o1 = p.costs[i]
-            o2 = q.costs[i]
-
-            # in artap we cannot change the direction of the optimization in this way
-            # if problem.directions[i] == Problem.MAXIMIZE:
-            #    o1 = -o1
-            #    o2 = -o2
-
-            epsilon = float(self.epsilons[i % len(self.epsilons)])
-            i1 = math.floor(o1 / epsilon)
-            i2 = math.floor(o2 / epsilon)
-
-            if i1 < i2:
-                dominate1 = True
-                if dominate2:
-                    return False
-            elif i1 > i2:
-                dominate2 = True
-                if dominate1:
-                    return False
-
-        if not dominate1 and not dominate2:
-            return True
-        else:
-            return False
-
     def compare(self, p: list, q: list):
+        """
+        Eps Dominance comparator, the implementation based on Platypus.
+        This operator works similarly like the normal, Dominance operator. However, it divides the current value with
+        the epsilons and selects which dominates the other. If they are equal it selects those one, which is closer to
+        the boundary.
+
+        :param p: solution p
+        :param q: solution q
+        :return:
+        """
 
         # first check constraint violation, the last item is the feasibility, which is a real number if its zero,
         # it means that the solution is feasible
@@ -782,28 +768,28 @@ class EpsilonDominance(Dominance):
             if p[-1] == 0:
                 return 1  # p dominates
             elif q[-1] == 0:
-                return 2  # q is dominates
-            elif p[-1] < q[-1]:
+                return 2  # q is dominates, because it has smaller degree in constraint violation
+            elif abs(p[-1]) < abs(q[-1]):
                 return 1  # p is dominates
-            elif q[-1] < p[-1]:
+            elif abs(q[-1]) < abs(p[-1]):
                 return 2  # q is dominates
 
         # then use epsilon dominance on the objectives
         dominate_p = False
         dominate_q = False
 
-        for i, (p_costs, q_costs) in enumerate(p[:-1], q[:-1]):
+        for i, (p_costs, q_costs) in enumerate(zip(p[:-1], q[:-1])):
 
             epsilon = float(self.epsilons[i % len(self.epsilons)])
 
-            pepsi = math.floor(p_costs / epsilon)
-            qepsi = math.floor(q_costs / epsilon)
+            p_eps = math.floor(p_costs / epsilon)
+            q_eps = math.floor(q_costs / epsilon)
 
-            if pepsi > qepsi:
+            if p_eps > q_eps:
                 dominate_q = True
                 if dominate_p:
                     return 0
-            elif qepsi > pepsi:
+            elif q_eps > p_eps:
                 dominate_p = True
                 if dominate_q:
                     return 0
@@ -812,16 +798,14 @@ class EpsilonDominance(Dominance):
             dist1 = 0.0
             dist2 = 0.0
 
-            for i in range(len(p.costs)):
-                o1 = p.costs[i]
-                o2 = q.costs[i]
-
+            for i, (p_costs, q_costs) in enumerate(zip(p[:-1], q[:-1])):
                 epsilon = float(self.epsilons[i % len(self.epsilons)])
-                i1 = math.floor(o1 / epsilon)
-                i2 = math.floor(o2 / epsilon)
 
-                dist1 += math.pow(o1 - i1 * epsilon, 2.0)
-                dist2 += math.pow(o2 - i2 * epsilon, 2.0)
+                i1 = math.floor(p_costs / epsilon)
+                i2 = math.floor(q_costs / epsilon)
+
+                dist1 += math.pow(p_costs - i1 * epsilon, 2.0)
+                dist2 += math.pow(q_costs - i2 * epsilon, 2.0)
 
             if dist1 < dist2:
                 return 1
@@ -832,6 +816,13 @@ class EpsilonDominance(Dominance):
         else:
             return 2
 
+    def same_box(self, p, q):
+        # if not dominate1 and not dominate2:
+        if self.compare(p, q) == 0:
+            return True
+        else:
+            return False
+
 
 class ParetoDominance(Dominance):
     """Pareto dominance with constraints.
@@ -856,22 +847,21 @@ class ParetoDominance(Dominance):
             if p[-1] == 0:
                 return 1  # p dominates
             elif q[-1] == 0:
-                return 2  # q is dominates
-            elif p[-1] < q[-1]:
+                return 2  # q is dominates, because it has smaller degree in constraint violation
+            elif abs(p[-1]) < abs(q[-1]):
                 return 1  # p is dominates
-            elif q[-1] < p[-1]:
+            elif abs(q[-1]) < abs(p[-1]):
                 return 2  # q is dominates
 
         dominate_p = False
         dominate_q = False
 
         for (p_costs, q_costs) in zip(p[:-1], q[:-1]):
-
             if p_costs > q_costs:
                 dominate_q = True
                 if dominate_p:
                     return 0
-            else:
+            elif q_costs > p_costs:
                 dominate_p = True
                 if dominate_q:
                     return 0
@@ -903,6 +893,33 @@ class Selector(Operator):
     def select(self, population):
         pass
 
+    def pop_acceptance(self, population, individual):
+        """
+        This function differs from add() function that it preserves the length of the _content in the archive class.
+        :return:
+        """
+
+        dominates = []
+        dominated = False
+
+        for i in range(len(population)):
+            flag = self.dominance.compare(individual.costs_signed, population[i].costs_signed)
+
+            if flag == 1:
+                dominates.append(i)
+            elif flag == 2:
+                dominated = True
+
+        if len(dominates) > 0:
+            del population[random.choice(dominates)]
+            population.append(individual)
+
+        elif not dominated:
+            population.remove(random.choice(population))
+            population.append(individual)
+
+        return
+
     def fast_nondominated_sorting(self, population):
 
         pareto_front = [[]]
@@ -915,12 +932,9 @@ class Selector(Operator):
             p.features['dominate'] = set()
 
         for i, p in enumerate(population):
-            # for q in population:
             for j in range(i + 1, len(population)):
                 q = population[j]
-                # if p is q:
-                #    continue
-                dom = self.comparator.compare(p.signs, q.signs)
+                dom = self.comparator.compare(p.costs_signed, q.costs_signed)
                 if dom == 1:
                     p.features['dominate'].add(q)
                     q.features['domination_counter'] += 1
@@ -942,9 +956,9 @@ class Selector(Operator):
                     if q.features['domination_counter'] == 0 and q.features['front_number'] is None:
                         q.features['front_number'] = front_number
                         pareto_front[front_number - 1].append(q)
-                        # temp_set.append(q)
-            # pareto_front = temp_set
-        pareto_front.pop()
+
+        if len(pareto_front[front_number - 1]) == 0:
+            pareto_front.pop()
         for sub_front in pareto_front:
             crowding_distance(sub_front)
 
@@ -960,12 +974,12 @@ def crowding_distance(front):
     """
     n = len(front)
 
-    if n is 0:
+    if n == 0:
         return
-    elif n is 1:
+    elif n == 1:
         front[0].features['crowding_distance'] = math.inf
         return
-    elif n is 2:
+    elif n == 2:
         front[0].features['crowding_distance'] = math.inf
         front[1].features['crowding_distance'] = math.inf
         return
@@ -973,16 +987,16 @@ def crowding_distance(front):
     for i in range(len(front)):
         front[i].features['crowding_distance'] = 0.0
 
-    for dim in range(len(front[0].signs[:-1])):
+    for dim in range(len(front[0].costs_signed[:-1])):
 
-        front.sort(key=lambda x: x.signs[dim])
+        front.sort(key=lambda x: x.costs_signed[dim])
         # self.sort_by_coordinate(population, dim)
 
         front[0].features['crowding_distance'] = math.inf
         front[-1].features['crowding_distance'] = math.inf
-        max_distance = front[-1].signs[dim] - front[0].signs[dim]
+        max_distance = front[-1].costs_signed[dim] - front[0].costs_signed[dim]
         for i in range(1, n - 1):
-            distance = front[i + 1].signs[dim] - front[i - 1].signs[dim]
+            distance = front[i + 1].costs_signed[dim] - front[i - 1].costs_signed[dim]
             if max_distance > 0.0:
                 front[i].features['crowding_distance'] += distance / max_distance
     return
@@ -1002,7 +1016,7 @@ def nondominated_truncate(population, size):
     """
 
     # calculating the crowding distance on the different fronts
-
+    population = list(set(population))
     result = sorted(population, key=functools.cmp_to_key(nondominated_cmp))
     return result[:size]
 
@@ -1069,7 +1083,13 @@ class TournamentSelector(Selector):
             # Sampling two individuals without a replacement
             candidates = random.sample(population, 2)
 
-            flag = self.dominance.compare(candidates[0].signs, candidates[1].signs)
+            # they should have this at this stage
+            if candidates[0].features['front_number'] < candidates[1].features['front_number']:
+                return candidates[0]
+            elif candidates[1].features['front_number'] < candidates[0].features['front_number']:
+                return candidates[1]
+
+            flag = self.dominance.compare(candidates[0].costs_signed, candidates[1].costs_signed)
 
             if flag == 1:
                 selected = candidates[0]
@@ -1081,78 +1101,79 @@ class TournamentSelector(Selector):
         return selected
 
 
-class Archive(object):
-    """An archive only containing non-dominated solutions."""
 
-    def __init__(self, signs, dominance=ParetoDominance):
-        super(Archive, self).__init__()
-        self._dominance = dominance(signs=signs)
-        self._contents = []
+# class Archive(object):
+#     """ Base archiving class based on platypus. An archive only containing non-dominated solutions."""
+#
+#     def __init__(self, signs, dominance=ParetoDominance):
+#         super(Archive, self).__init__()
+#         self._dominance = dominance(signs=signs) # dominance comparator
+#         self._contents = []
+#
+#     def add(self, solution):
+#         flags = [self._dominance.compare(solution, s) for s in self._contents]
+#         dominates = [x > 0 for x in flags]
+#         nondominated = [x == 0 for x in flags]
+#
+#         if any(dominates):
+#             return False
+#         else:
+#             self._contents = list(itertools.compress(self._contents, nondominated)) + [solution]
+#             return True
+#
+#     def append(self, solution):
+#         self.add(solution)
+#
+#     def extend(self, solutions):
+#         for solution in solutions:
+#             self.append(solution)
+#
+#     def remove(self, solution):
+#         try:
+#             self._contents.remove(solution)
+#             return True
+#         except ValueError:
+#             return False
+#
+#     def __len__(self):
+#         return len(self._contents)
+#
+#     def __getitem__(self, key):
+#         return self._contents[key]
+#
+#     def __iadd__(self, other):
+#         if hasattr(other, "__iter__"):
+#             for o in other:
+#                 self.add(o)
+#         else:
+#             self.add(other)
+#
+#         return self
+#
+#     def __iter__(self):
+#         return iter(self._contents)
 
-    def add(self, solution):
-        flags = [self._dominance.compare(solution, s) for s in self._contents]
-        dominates = [x > 0 for x in flags]
-        nondominated = [x == 0 for x in flags]
-
-        if any(dominates):
-            return False
-        else:
-            self._contents = list(itertools.compress(self._contents, nondominated)) + [solution]
-            return True
-
-    def append(self, solution):
-        self.add(solution)
-
-    def extend(self, solutions):
-        for solution in solutions:
-            self.append(solution)
-
-    def remove(self, solution):
-        try:
-            self._contents.remove(solution)
-            return True
-        except ValueError:
-            return False
-
-    def __len__(self):
-        return len(self._contents)
-
-    def __getitem__(self, key):
-        return self._contents[key]
-
-    def __iadd__(self, other):
-        if hasattr(other, "__iter__"):
-            for o in other:
-                self.add(o)
-        else:
-            self.add(other)
-
-        return self
-
-    def __iter__(self):
-        return iter(self._contents)
-
-
-class EpsilonBoxArchive(Archive):
-
-    def __init__(self, epsilons):
-        super(EpsilonBoxArchive, self).__init__(EpsilonDominance(epsilons))
-        self.improvements = 0
-
-    def add(self, solution):
-        flags = [self._dominance.compare(solution, s) for s in self._contents]
-        dominates = [x > 0 for x in flags]
-        nondominated = [x == 0 for x in flags]
-        dominated = [x < 0 for x in flags]
-        not_same_box = [not self._dominance.same_box(solution, s) for s in self._contents]
-
-        if any(dominates):
-            return False
-        else:
-            self._contents = list(itertools.compress(self._contents, nondominated)) + [solution]
-
-            if dominated and not_same_box:
-                self.improvements += 1
+#
+# class EpsilonBoxArchive(Archive):
+#
+#     def __init__(self, epsilons):
+#         super(EpsilonBoxArchive, self).__init__(EpsilonDominance(epsilons))
+#         self.improvements = 0
+#
+#     def add(self, solution):
+#         flags = [self._dominance.compare(solution, s) for s in self._contents]
+#         dominates = [x > 0 for x in flags]
+#         nondominated = [x == 0 for x in flags]
+#         dominated = [x < 0 for x in flags]
+#         not_same_box = [not self._dominance.same_box(solution, s) for s in self._contents]
+#
+#         if any(dominates):
+#             return False
+#         else:
+#             self._contents = list(itertools.compress(self._contents, nondominated)) + [solution]
+#
+#             if dominated and not_same_box:
+#                 self.improvements += 1
 
 
 class Crossover(Operator):
@@ -1201,7 +1222,7 @@ class SimpleCrossover(Crossover):
 
 class SimulatedBinaryCrossover(Crossover):
 
-    def __init__(self, parameters, probability, distribution_index=20):
+    def __init__(self, parameters, probability, distribution_index=15):
         super().__init__(parameters, probability)
         self.distribution_index = distribution_index
         if distribution_index < 0:
