@@ -2,17 +2,12 @@ from .individual import Individual
 
 
 class Population:
-    def __init__(self, individuals: list = None, archives: list = None):
+    def __init__(self, individuals: list = None):
         if individuals is None:
             self.individuals = []
         else:
             self.individuals = individuals.copy()
 
-        if archives is None:
-            self.archives = []
-        else:
-            self.archives = archives.copy()
-
         self.pareto_vectors: list = []
         self.pareto_costs: list = []
 
@@ -26,4 +21,4 @@ class Population:
         if not isinstance(other, Population):
             return NotImplemented
 
-        return self.individuals == other.individuals and self.archives == other.archives and self.pareto_vectors == other.pareto_vectors and self.pareto_costs == other.pareto_costs
\ No newline at end of file
+        return self.individuals == other.individuals and self.pareto_vectors == other.pareto_vectors and self.pareto_costs == other.pareto_costs
\ No newline at end of file
