@@ -89,12 +89,14 @@ class Problem:
         self.description: str = str()
         self.parameters: dict = dict()
         self.costs: dict = dict()
+        self.signs = []
 
         # populations
         self.populations = []
+        self.archive = None
 
         self.data_store = None
-        self.monitor_service = MonitorService(self)
+        self.monitor_service = None
         self.executor = None
 
         self.output_files = None
@@ -104,7 +106,6 @@ class Problem:
 
         # surrogate model (default - only simple eval)
         self.surrogate = SurrogateModelEval(self)
-        self.signs = []
 
         # self._freeze()
         self.set(**kwargs)
@@ -124,10 +125,11 @@ class Problem:
         if self.data_store:
             self.data_store.destroy()
 
-        if self.monitor_service.server is not None:
-            self.monitor_service.server.close()
-            del self.monitor_service.server
-            self.monitor_service.server = None
+        if self.monitor_service is not None:
+            if self.monitor_service.server is not None:
+                self.monitor_service.server.close()
+                del self.monitor_service.server
+                self.monitor_service.server = None
 
         if self.working_dir.startswith(tempfile.gettempdir() + os.sep + "artap-"):
             shutil.rmtree(self.working_dir)
