@@ -163,10 +163,10 @@ class Results:
         # if len(self.problem.data_store.individuals) is not 0:
         #    min_l = [min(self.problem.data_store.individuals, key=lambda x: x.costs[index])]
         # else:
-        if len(self.problem.populations[-1].archives) > 0:
-            min_l = [min(self.problem.populations[-1].archives, key=lambda x: x.costs[index])]
+        if self.problem.archive:
+            min_l = [min(self.problem.archive, key=lambda x: x.costs[index])]
         else:
-            if len(self.problem.populations[-1].individuals) is not 0:
+            if len(self.problem.populations[-1].individuals) != 0:
                 min_l = [min(self.problem.populations[-1].individuals, key=lambda x: x.costs[index])]
         # for population in self.problem.populations:
         opt = min(min_l, key=lambda x: x.costs[index])
@@ -200,9 +200,9 @@ class Results:
 
         computed = self.pareto_values()  # [[], [], .. ]
 
-        if type is 'epsilon':
+        if type == 'epsilon':
             result = epsilon_add(reference, computed)
-        if type is 'gd':
+        if type == 'gd':
             result = gd(reference, computed)
 
         return result
@@ -248,14 +248,15 @@ class Results:
         population = self.problem.populations[-1]
         l_sol = []
 
-        if len(self.problem.populations[-1].archives) < 1:
+        # the pareto values collected in the archive, if the algorithm uses this strategy
+        if self.problem.archive:
+            for individual in self.problem.archive:
+                l_sol.append(individual.costs)
+        else:
             if len(population.individuals) > 1:
                 for individual in population.individuals:
                     l_sol.append(individual.costs)
-        else:
-            if len(population.archives) > 1:
-                for individual in population.archives:
-                    l_sol.append(individual.costs)
+
         return l_sol
 
     def parameters(self):
